==> openturns_0.12.2.orig.tar.gz: openturns-0.12.2/lib/src/Uncertainty/Distribution/BetaFactory.cxx <==
//                                               -*- C++ -*-
/**
 *  @file  BetaFactory.cxx
 *  @brief Factory for Beta distribution
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2007-05-10 16:43:31 +0200 (jeu 10 mai 2007) $
 *  Id:      $Id: BetaFactory.cxx 434 2007-05-10 14:43:31Z dutka $
 */
#include "BetaFactory.hxx"
#include "Beta.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Distribution {

      /* Default constructor */
      BetaFactory::BetaFactory()
      {
	// Initialize any other class members here
	// At last, allocate memory space if needed, but go to destructor to free it
      }

      /* Virtual constructor */
      BetaFactory * BetaFactory::clone() const
      {
	return new BetaFactory(*this);
      }


      /* Here is the interface that all derived class must implement */

      BetaFactory::DistributionImplementation * BetaFactory::buildImplementation(const NumericalSample & sample) const throw(InvalidArgumentException, InternalException)
      {
	if (sample.getSize() == 0) throw InvalidArgumentException(HERE) << "Error: cannot build a Beta distribution from an empty sample";
	if (sample.getDimension() != 1) throw InvalidArgumentException(HERE) << "Error: can build a Beta distribution only from a sample of dimension 1, here dimension=" << sample.getDimension();
	NumericalScalar a(sample.getMin()[0]);
	NumericalScalar b(sample.getMax()[0]);
	// Correction for bias control
	NumericalScalar size(sample.getSize());
	if (a > 0.0)
	  {
	    a *= 1.0 - 1.0 / (2.0 + size);
	  }
	else
	  {
	    a *= 1.0 + 1.0 / (2.0 + size);
	  }
	if (b > 0.0)
	  {
	    b *= 1.0 + 1.0 / (2.0 + size);
	  }
	else
	  {
	    b *= 1.0 - 1.0 / (2.0 + size);
	  }
	if (a >= b) throw InternalException(HERE) << "Error: can build a Beta distribution only if a < b, here a=" << a << " and b=" << b;
	NumericalScalar mean(sample.computeMean()[0]);
	NumericalScalar sigma(sample.computeStandardDeviationPerComponent()[0]);
	NumericalScalar t((b - mean) * (mean - a) / (sigma * sigma) - 1.0);
	NumericalScalar r(t * (mean - a) / (b - a));
	return Beta(r, t, a, b).clone();
      }


    } /* namespace Distribution */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

==> openturns_0.13.1.orig.tar.gz: openturns-0.13.1/lib/src/Uncertainty/Distribution/BetaFactory.cxx <==
//                                               -*- C++ -*-
/**
 *  @file  BetaFactory.cxx
 *  @brief Factory for Beta distribution
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2009-05-28 10:54:56 +0200 (jeu. 28 mai 2009) $
 *  Id:      $Id: BetaFactory.cxx 1261 2009-05-28 08:54:56Z dutka $
 */
#include "BetaFactory.hxx"
#include "Beta.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Distribution {

      /* Default constructor */
      BetaFactory::BetaFactory()
      {
	// Initialize any other class members here
	// At last, allocate memory space if needed, but go to destructor to free it
      }

      /* Virtual constructor */
      BetaFactory * BetaFactory::clone() const
      {
	return new BetaFactory(*this);
      }


      /* Here is the interface that all derived class must implement */

      BetaFactory::DistributionImplementation * BetaFactory::buildImplementation(const NumericalSample & sample) const throw(InvalidArgumentException, InternalException)
      {
	if (sample.getSize() == 0) throw InvalidArgumentException(HERE) << "Error: cannot build a Beta distribution from an empty sample";
	if (sample.getDimension() != 1) throw InvalidArgumentException(HERE) << "Error: can build a Beta distribution only from a sample of dimension 1, here dimension=" << sample.getDimension();
	NumericalScalar a(sample.getMin()[0]);
	NumericalScalar b(sample.getMax()[0]);
	// Correction for bias control
	NumericalScalar size(sample.getSize());
	if (a > 0.0)
	  {
	    a *= 1.0 - 1.0 / (2.0 + size);
	  }
	else
	  {
	    a *= 1.0 + 1.0 / (2.0 + size);
	  }
	if (b > 0.0)
	  {
	    b *= 1.0 + 1.0 / (2.0 + size);
	  }
	else
	  {
	    b *= 1.0 - 1.0 / (2.0 + size);
	  }
	if (a >= b) throw InternalException(HERE) << "Error: can build a Beta distribution only if a < b, here a=" << a << " and b=" << b;
	NumericalScalar mean(sample.computeMean()[0]);
	NumericalScalar sigma(sample.computeStandardDeviationPerComponent()[0]);
	NumericalScalar t((b - mean) * (mean - a) / (sigma * sigma) - 1.0);
	NumericalScalar r(t * (mean - a) / (b - a));
	return Beta(r, t, a, b).clone();
      }

      BetaFactory::DistributionImplementation * BetaFactory::buildImplementation(const NumericalPointCollection & parameters) const throw(InvalidArgumentException, InternalException)
      {
	try {
	  Beta distribution;
	  distribution.setParametersCollection(parameters);
	  return distribution.clone();
	}
	catch (InvalidArgumentException & ex)
	  {
	    throw InternalException(HERE) << "Error: cannot build a Beta distribution from the given parameters";
	  }
      }

      BetaFactory::DistributionImplementation * BetaFactory::buildImplementation() const throw(InvalidArgumentException, InternalException)
      {
	return Beta().clone();
      }


    } /* namespace Distribution */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */
